/**************************************************************************
 *
 * socket.h
 *
 * Written by Shaun Terry (72010.1771@compuserve.com).
 * Copyright 1995. Freeware.
 *
 **************************************************************************
 */

#ifndef _SOCKET_H
#define _SOCKET_H

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netdb.h>
#include <netinet/in.h>
#include <netinet/tcp.h>

#ifdef _HPUX_SOURCE
#define getdtablesize()		NOFILE
#endif

const int InvalidSocket = (-1);


class Socket {

public:
	// Type for status callback 
	typedef void (*SocketCallback)(
	    Socket *, 	// Pointer to class object
	    void *);		// Client data passed from invoker

private:
	// Function to call when socket dies
	SocketCallback deadSocketCB;
	void *deadSocketClientData;

	// Socket file descriptor
	int sock;
	int portNumber;

public:
	Socket();
	Socket(int fd) { sock = fd; }
	~Socket();

	// Open after instantiation via host name and port number 
	int openHost(char *hostName, int portNum);

	// Open after instantiation via host name and NIS service name
	int openHost(char *hostName, char *service);

	// Normal Close
	int close();

	// Close with drainage
	int niceClose();

	// Blocking send
	int send( 
		char *message,		// Pointer to message to transmit
		int length);		// lenght of the message in bytes

	// Blocking read
	int recv( 
		char *message, 		// Pointer to memory to receive into
		int length);		// Maximum willing receive size

	// Server type listen on port number
	int hostListen(int port, int maxConnections =5);
	int hostListen(char *serviceName, int maxConnections =5);

	// Server accept connection
	int accept(sockaddr_in *from =NULL, int *len =NULL);

	// Set callback if socket is closed by other side 
	void setDeadSocketCB(SocketCallback cb, void *cl =NULL) {
		deadSocketCB = cb;
		deadSocketClientData = cl;
	}

	// Get the socket file descriptor
	int sockFd() { return(sock); }

	// Get the (possibly system-allocated) port number
	int port() { return portNumber; }

	// General-purpose port number query by socket fd
	int getPortOfSocket(int sockId);
};

extern "C" {
	char *inet_ntoa(struct in_addr);
#ifdef sun4
	int gethostname(char *, int);
#endif
	unsigned long inet_addr(char *);
}

#endif
 
